<?php
namespace Folha\Logic\ESocial\Eventos\Data;
use Folha\Logic\ESocial\GerenciaEnvios;
use HCM\Util\DateUtil;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1050 - Tabela de Horarios/Turnos de Trabalho.
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author paulopereira
 **/
class S1050
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1050, Tabela de Horarios/Turnos 
     * de Trabalho.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author paulopereira
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente)
    {
        $S1050 = \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S1050(
            $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
            $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente
        );

        foreach ($S1050 as &$escala) {
            /*Verifica pelo log de envios se o evento/chave já foi enviado para definir ser é um envio de inclusão ou alteração*/
            $operacao = new GerenciaEnvios('S-1050', $versaoEsocial, $escala['CHAVE'], null, $nrorg);
            $operacao = $operacao->validaTipoEnvio();

            $escala['INCLUSAO'] = $operacao['INCLUSAO'];
            $escala['ALTERACAO'] = $operacao['ALTERACAO'];

            if ($versaoEsocial === '2.0') {
                $horarioIntervalo = null;
                $periodos = \Folha\Repositories\GpeEsocialRegistro::s1050_horarioIntervalo($escala['NRHORDIAM'], $nrorg);

                $seqPerMaior = null;
                $iniPerMaior = null;
                $fimPerMaior = null;

                foreach ($periodos as $periodo) {

                    if (is_null($seqPerMaior)) {
                        $seqPerMaior = $periodo['NRSEQINTERVHORARIO'];
                        $iniPerMaior = $periodo['HRENTRADA'];
                        $fimPerMaior = $periodo['HRSAIDA'];
                    }
                    else {
                        $seqPerMenor = $periodo['NRSEQINTERVHORARIO'];
                        $iniPerMenor = $periodo['HRENTRADA'];
                        $fimPerMenor = $periodo['HRSAIDA'];

                        $durInterv = DateUtil::subtrairHoras($iniPerMaior, $fimPerMenor);
                        $durIntervHor = DateUtil::getHoraFloatDeString($durInterv);
                        $durIntervMin = $durIntervHor * 60;
                        $horarioIntervalo[] =
                            $escala['INCLUSAO'] == 1 ?
                                array(
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_INCLUSAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_TPINTERV' => $periodo['TPINTERV'],
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_INCLUSAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_DURINTERV' => $durIntervMin,
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_INCLUSAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_INIINTERV' => $fimPerMenor,
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_INCLUSAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_TERMINTERV' => $iniPerMaior
                                )
                            :
                                array(
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_ALTERACAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_TPINTERV' => $periodo['TPINTERV'],
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_ALTERACAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_DURINTERV' => $durIntervMin,
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_ALTERACAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_INIINTERV' => $fimPerMenor,
                                    '_EVTTABHORTUR_INFOHORCONTRATUAL_ALTERACAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_TERMINTERV' => $iniPerMaior
                                );

                        $seqPerMaior = $seqPerMenor;
                        $iniPerMaior = $iniPerMenor;
                        $fimPerMaior = $fimPerMenor;
                    }
                }

                $escala['HORARIOINTERVALO'] = $horarioIntervalo;
            }
            else {
                /*Preenche os campos do horário de intervalo*/
                $escala['HORARIOINTERVALO'][0] =
                    array("_EVTTABHORCONTRATUAL_INFOHORCONTRATUAL_INCLUSAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_INICIOINTERVALO" => $escala['INICIOINTERVALO'],
                        "_EVTTABHORCONTRATUAL_INFOHORCONTRATUAL_INCLUSAO_DADOSHORCONTRATUAL_HORARIOINTERVALO_TERMINOINTERVALO" => $escala['TERMINOINTERVALO']);
            }
        }

        return $S1050;
    }

}